//
// Gaggle textures
// by Jarno van der Linden  jvan006@cs.auckland.ac.nz
//
// Image loading is courtesy of the SDL_image library by Sam Lantinga
//

#include "gaggle.h"

#include <iostream>
#include <fstream>

#include <cmath>

extern "C"
{
#include "SDL_image.h"
}


using namespace Gaggle;
using namespace std;


//
// A Texture
//

Texture::Texture(const string& filename)
	: ModelData(filename),
	  width(0), height(0), texture(0)
{
	DecompressFromMem(GetCompressedData(), GetCompressedSize());
}

Texture::Texture(const TextureMessage& msg)
	: ModelData(msg, sizeof(msg)),
	  width(0), height(0), texture(0)
{
	DecompressFromMem(GetCompressedData(), GetCompressedSize());
}


Texture::~Texture()
{
	delete[] texture;
}


bool Texture::DecompressFromMem(const unsigned char* data, const size_t datasize)
{
	SDL_RWops *rwops = SDL_RWFromMem((void *)data, datasize);
	SDL_Surface *image = IMG_Load_RW(rwops, 1);

	width = height = 0;
	delete[] texture;
	texture = NULL;

	if(!image)
		return false;

	width = image->w;
	height = image->h;
	texture = new unsigned char[width * height * 3];
	for(int y = 0; y < height; y++)
	{
		memcpy(&(texture[y * width * 3]), &(((unsigned char *)image->pixels)[y * image->pitch]), width*3);
	}
	SDL_FreeSurface(image);

	return true;
}


void Texture::GetDimension(int& width_out, int& height_out) const
{
	width_out = width;
	height_out = height;
}


// A message containing a texture
TextureMessage* TextureMessage::Create(const Texture& texture)
{
	TextureMessage *texturemsg = (TextureMessage *)ModelDataMessage::Create(texture, ID_TEXTURE_MESSAGE, sizeof(TextureMessage));
	return texturemsg;
}


// A message requesting a texture
void AskTextureMessage::Fill(AskTextureMessage& msg, UID uid)
{
	GaggleMessage::Fill(msg, ID_ASKTEXTURE_MESSAGE, sizeof(AskTextureMessage));
	msg.uid = uid;
}
